
from django.contrib.auth.models import User

from django.urls import reverse_lazy

from django.core.exceptions import PermissionDenied

from django.contrib.auth.views import redirect_to_login

from django.contrib.auth.mixins import LoginRequiredMixin, UserPassesTestMixin

from django.views.generic import (
    ListView,CreateView,DetailView, UpdateView,DeleteView
)

from django.shortcuts import render, get_object_or_404

from blog.models import Post

'''
def home_view(request):
    posts = Post.objects.all().order_by('-date_posted')

    context = {
        'posts': posts
    }

    return render(request, 'blog/post_list.html', context)
'''


class PostListView(ListView):
    model = Post
    context_object_name = 'posts'
    template_name = 'blog/post_list.html'
    ordering = ['-date_posted']
    paginate_by = 2


class PostDetailView(DetailView):
    model = Post
    context_object_name = 'post'
    template_name = "blog/post_detail.html"


class PostCreateView(LoginRequiredMixin, CreateView):
    model = Post
    fields = ['title', 'content']
    context_object_name = 'form'
    template_name = "blog/post_form.html"

    def form_valid(self, form):
        form.instance.author = self.request.user
        return super().form_valid(form)


class PostUpdateView(LoginRequiredMixin, UserPassesTestMixin, UpdateView):
    model = Post
    fields = ['title', 'content']
    context_object_name = 'form'
    template_name = "blog/post_form.html"
    login_url = "/user/login/"

    def form_valid(self, form):
        form.instance.author = self.request.user
        return super().form_valid(form)

    def test_func(self):
        post = self.get_object()
        if self.request.user == post.author:
            return True
        return False

    def handle_no_permission(self):
        if self.raise_exception:
            raise PermissionDenied(self.get_permission_denied_message())
        return redirect_to_login(
            self.request.get_full_path(),
            self.get_login_url(),
            self.get_redirect_field_name()
        )


class PostDeleteView(LoginRequiredMixin, UserPassesTestMixin, DeleteView):
    model = Post
    context_object_name = 'post' # 'object'
    template_name = "blog/post_confirm_delete.html"
    login_url = "/user/login/"

    # for deleting this success_url is required

    # success_url = reverse_lazy('/fdfds') # before deleting the record, first searching for valid path

    # success_url = reverse_lazy('blog:home_page')

    success_url = '/'  # First deleting record and then redirecting to given url path

    def test_func(self):
        post = self.get_object()
        if self.request.user == post.author:
            return True
        return False

    def handle_no_permission(self):
        if self.raise_exception:
            raise PermissionDenied(self.get_permission_denied_message())
        return redirect_to_login(
            self.request.get_full_path(),
            self.get_login_url(),
            self.get_redirect_field_name()
        )



class UserPostListView(ListView):
    model = Post
    context_object_name = 'user_posts'
    template_name = 'blog/user_post_list.html'
    ordering = ['-date_posted']
    paginate_by = 2

    def get_queryset(self):
        user = get_object_or_404(User, username=self.kwargs.get('username'))
        return Post.objects.filter(author=user)

        # get_object_or_404(User, username=self.request.user.username)



def latest_posts_view(request):
    latest_posts = Post.objects.all().order_by('-date_posted')[:3]
    context = {
        'latest_posts': latest_posts
    }
    return render(request, 'blog/latest_posts.html', context)


def interview_questions_view(request):
    return render(request, 'blog/interview_questions.html')


def responsive_view(request):
    return render(request, 'blog/responsive.html')







def about_view(request):
    return render(request, 'blog/about.html')
